/*
=====================================================
SIMSIG.AHK
v1.0, 2011-12-09
Tested on SimSig system files v2.223.1 running on Windows 7
Uses AutoHotkey Basic v1.0.48.05 2009-09
Language:       English
Platform:       Win9x/NT/XP/Vista/7
Author:         Max Anderson
Contact:        maxxand@fastmail.fm

Disclaimer:
Use these macros at your own risk. I accept no responsibility
for any damage this code may do to your system.

Feel free to modify this script for your own use.
This script and accompanying documentation is not
for sale or commercial distribution.
=====================================================
*/

;--------------------------
;AUTO-EXECUTE SECTION (AES)
;--------------------------

;All code from here on will be executed automatically,
;until AHK encounters the first hotkey or hotstring (i.e., the first macro).
;Consider the AES to be initialization code.

;----------------------------------------
;Initialization code common to ALL macros
;----------------------------------------
;(1)Standard Environment Settings
;Note: You can paste part or all of this section into "ShellNew\Template.ahk"
;in your Windows folder and make it part of your template.

#NoEnv  ;Recommended for performance and compatibility with future AutoHotkey releases.
SendMode Input  ;Recommended for new scripts due to its superior speed and reliability.
SetWorkingDir %A_ScriptDir%  ;Ensures a consistent starting directory.

;(2)Custom Settings for all macros
SetTitleMatchMode, 1 ;A window's title must *start* with the specified WinTitle to be a match.
DetectHiddenWindows, Off ;Default is Off

;----------------------------------------
;Initialization code for particular macros
;----------------------------------------

/*
Initialize any global variables (called by auto-execute section, see top of script)
Plus any other startup routines
*/

/* Create SSGroup
When a macro has to work no matter which SimSig window happens to be foremost,
it simplifies code to create a Group of windows that can be referred to collectively
in one statement. These commands add most SimSig windows to a group named SSGroup.
Exceptions include Options (F3)
*/
GroupAdd, SSGroup, SimSig
GroupAdd, SSGroup, View
GroupAdd, SSGroup, Overview
GroupAdd, SSGroup, Train List
GroupAdd, SSGroup, Timetable List
GroupAdd, SSGroup, Show Timetable

MsgBoxClosing := 2 ;No. of seconds after which self-closing MsgBoxes close

Gosub, M16_InitGlobalVars
  ;Sets initial transparency level

;------------------------------
; MACROS begin here
;------------------------------

;------------------------------
; MISCELLANEOUS MACROS
;------------------------------
;#M01 Display Macro Menu
;Please remember to upgrade this menu if changing hotkeys
#IfWinActive, ahk_group SSGroup
M::
  MsgBox,262144,Macro Menu,
    (LTrim
    FUNCTION KEYS
    F1: Toggle Show Timetable window
    F2: Toggle Train List window
    F5:  Toggle Overview window
    F12: Display Messages window
    Ctrl+F12:  Check that SimSig is loaded and on top

    VIEW WINDOW
    Shift+LClick: Send Double-Click
    Ctrl+RClick:  Cancel Route
    Win+RClick: Interpose at signal / Cancel Interpose
    C: Copy TD to Windows Clipboard
    Alt+C: Copy last 3 TD chars to Clipboard

    ALTERNATIVE SHORTCUTS
    S,F,D,R,T: Left Hand cursor keys
    Alt+F: Fast
    Alt+W: Pause ("Wait")

    STICKY NOTES
    V: Create Sticky note with Clipboard and leave open
    B: Create Sticky note with Clipboard and close
    Alt+Z: Left Hand Close Sticky Note
    Alt+D: Delete Sticky Note

    TRANSPARENCY LEVELS (Train List, Show Timetable)
    Ctrl+Shift+Minus: Less opaque (more transparent)
    Ctrl+Shift+Plus: More opaque (less transparent)
    Ctrl+Shift+PrtSc/SysRq: Reset to initial value
    Ctrl+Shift+Pause/Break: Set to fully opaque


    M: Display this Menu
    )
Return

;------------------------------
;#M02 Check that SimSig is loaded and active
;Keys: Ctrl+F12
^F12::
  IfWinNotExist, SimSig
    MsgBox SimSig must be loaded before these macros will work.
  else
    {
    WinGetActiveTitle, topWin ;store title of currently active window
    topWin := SubStr(topWin,1,(InStr(topWin,A_Space,1)-1))
      ;Trim off the first space (if any) and any subsequent characters
      ;MsgBox topwin is "%topWin%" ;leave this in to test
    If topWin not in View ;add other SimSig windows to this list if desired
      MsgBox, 0, SimSig View not active, Please click on SimSig's View window `n to activate it before running macros.
    else
      MsgBox, 0, SimSig View active, SimSig loaded and active - macros should work OK.
    }
Return

;------------------------------
; SUBWINDOW MACROS
;------------------------------
;#M03 Display Messages window
;Useful if partially or completely hidden by View window
;Assumes it is already created, which is SimSig default
;Keys: F12
F12::
  ;For this and other macros which depend on SimSig's Control window
  ;menu functions, this window must be open. The Control window
  ;is titled "Simsig - [sim name]".
  Gosub, glbActivateControlWindow ;Activate Control window for its menu
  SendInput !sm ;Show > Messages
Return

;------------------------------
;#M04 Toggle Overview window
;Keys: F5 (was Performance Analysis)
F5::
  IfWinNotExist, Overview ;if user has not displayed it yet
    {
    Gosub, glbActivateControlWindow ;Activate Control window for its menu
    SendInput !sv ;create Overview window
    Return
    }
  else ;Overview window exists but may be hidden
    {
    IfWinNotActive, Overview ;if hidden, bring it to front
      WinActivate, Overview
    else
      WinActivate, View  ;bring View to front if Overview overlapped it
    }
Return

;------------------------------
;#M05 Toggle Train List window
;Enables your current window size settings saved,
;and no hunting for the Restore Up icon
;Keys: F2
#IfWinActive, ahk_group SSGroup ;Enables this macro no matter which SS window is on top
F2::
  IfWinNotExist, Train List ;if user has not asked for it before
    {
    WinActivate, View ;F2 won't work unless SimSig or View window is active
    SendInput {F2} ;create Train List
    WinSet, Transparent, %M16CurrTrnsp%, Train List ;set transparency level
    Return
    }
  else ;Train List window exists but may be minimized
    {
    WinGet, win_state, MinMax, Train List ;test min/normal/max status
    If (win_state = -1) ;if minimized
      WinRestore, Train List  ;restore to normal
      ;It should automatically get focus; if not, add [WinActivate, View]
    else ;if normal or maximized
      {
      WinMinimize, Train List ;minimize it
      WinActivate, View ;move focus to View window (necessary)
      }
    }
Return

;------------------------------
;#M06 Toggle Show Timetable window
;Enables your current window size settings saved,
;and no hunting for the Restore Up icon
;Keys: F1
#IfWinActive, ahk_group SSGroup ;Enables this macro no matter which SS window is on top
F1::
  IfWinNotExist, Show Timetable ;if user has not displayed it yet
    {
    MsgBox,,No "Show Timetable" window,
          (LTrim
          First left-click a Train ID to show timetable

          (closing in %MsgBoxClosing% seconds...)
          ),%MsgBoxClosing%
    WinActivate, View ;return to View window
    Return
    }
  else ;Show Timetable window exists but may be minimized
    {
    WinGet, win_state, MinMax, Show Timetable ;test min/normal/max status
    If (win_state = -1) ;if minimized
      WinRestore, Show Timetable  ;restore to normal
      ;It should automatically get focus; if not, add [WinActivate, View]
    else
      {
      WinMinimize, Show Timetable ;minimize it
      WinActivate, View ;move focus to View window
          ;Note: this will hide Overview window if View window overlaps it
      }
    }
Return

;------------------------------
; VIEW WINDOW MACROS
;-------------------------------
;#M07 Send Double-Click
;Ends setting one route and begins setting another
;Keys: Shift+LClick (hovering over signal)
#IfWinActive, View
+LButton::Click 2
Return

;------------------------------
;#MO8 Cancel Route
;Enables R-click to display signal number, yet offers
;direct hotkey to cancel route
;Assumes option "Right click cancels route" is unchecked
;Keys: Ctrl+RClick (hovering over signal)
#IfWinActive, View
^RButton::
  Click right
  SendInput {Down 2} {Enter}
Return

;------------------------------
;#M09 Interpose at signal (or Cancel TD)
;If no train at signal, it interposes TD
;If train at signal, same menu position cancels TD
;Keys: Win+RClick (hovering over signal)
#IfWinActive, View
#RButton::
  Click right
  SendInput {Up 2}
  SendInput {Enter}
Return

;------------------------------
;#M10 Copy TD to Windows Clipboard
;Keys: C (hovering over train berth)
#IfWinActive, View
C::
  Gosub, glbCopyTDToClipboard
Return

;------------------------------
;#M11 Copy last 3 TD chars to Clipboard
;Useful for changing to "next working" TD in London Underground trains
;Keys: Alt+C
#IfWinActive, View
!C::
  Gosub, glbCopyTDToClipboard
  StringTrimLeft,Clipboard, Clipboard,1
Return

;------------------------------
; ALTERNATIVE SHORTCUTS
;------------------------------
;#M12a LH Cursor Keys
;Keys: S,F,D,R,T (can't use traditional A,D as A is too useful)
;F key needs to be re-mapped (see next macro)
;Feel free to change these to any combinating that suits your left hand
#IfWinActive, View
S:: SendInput {Left 5}  ;gives wider leap than sending it once
F:: SendInput {Right 5} ;ditto

/*
These are possible hotkey contenders for LH Up/Down but not in use at present
E:: SendInput {Up}
X:: SendInput {Down}
*/

;------------------------------
;#M12b Sim-specific LH Cursor Keys

/*
Add your own sim settings here
E.g., if [sim name] has 5 views (1-5), insert this code:

#IfWinActive, View - [sim name] ;[sim name] has 5 view settings
D:: SendInput 3  ;middle view
R:: SendInput 1  ;leftmost view
T:: SendInput 5  ;rightmost view

*/

#IfWinActive, View - Exeter ;Exeter has 9 view settings
D:: SendInput 5  ;middle view
R:: SendInput 1  ;leftmost view
T:: SendInput 9  ;rightmost view

;------------------------------
;#M12c Alternative Fast
;In this script, F is used to scroll View right,
;Keys: Alt+F
#IfWinActive, View
!F::
#IfWinActive, SimSig
!F::
  SendInput f ;MUST be lower case
Return

;------------------------------
;#M12d Alternative LH Pause
;Easier than reaching across keyboard to press P
;if RH is already on mouse - it enables you
;to keep RH on mouse!
;Keys: Alt+W ("Wait")
#IfWinActive, ahk_group SSGroup
!W::
  SendInput p ;MUST be lower case
Return

;------------------------------
; STICKY NOTE MACROS
;------------------------------
;#M15a Create Sticky note with Clipboard and leave open
;Keys: V (hovering over desired Sticky Note position in background of View window)
;V was chosen as it is similar to Ctrl+V, cf. C and Ctrl+C
#IfWinActive, View
V::
  Gosub, glbCreateStickyNoteWithClipboard
  SendInput ^v %A_Space% ;paste contents of clipboard (e.g TD) and add 1 space
  ;Leave Sticky Note open for further editing
Return

;------------------------------
;#M15b Create Sticky note with Clipboard and close
;Keys: B (hovering over desired Sticky Note position in background of View window)
#IfWinActive, View
B::
  Gosub, glbCreateStickyNoteWithClipboard
  SendInput ^v ;paste contents of clipboard (e.g TD)
  SendInput !o ;click OK to close sticky note
Return

;------------------------
;#M15c LH Close Sticky Note
;One-handed alternative to using both hands to type Alt+O
;Keys: Alt+Z
#IfWinActive, Edit Sticky Note ;this window must be on top to enable macro
!Z::SendInput !o ;This accepts edits and closes window; Alt+F4 cancels & closes
Return

;----------------------
;#M15d Delete Sticky Note
;Currently this can only be done using the mouse, and while Sticky Note window is open
;Keys: Alt+D (hovering over sticky note window)
#IfWinActive, View
!D::
  CoordMode, Mouse, Screen
  MouseGetPos, xPos, yPos ;Store mouse coordinates
    ;When sticky note is deleted, cursor will remain at "Delete note" button position
    ;in View window unless coordinates are restored
  Click right ;open sticky note window
  WinWaitActive, Edit Sticky Note ;sticky note window must be on top
  CoordMode, Mouse, Relative ;click coordinates relative to active window
                             ;thankfully Sticky Note window's size is fixed
  Click 135,35               ;click "Delete note" button; don't try to delete actual window
  CoordMode, Mouse, Screen
  MouseMove, xPos, yPos ;Restore mouse coordinates
Return

;----------------------
; TRANSPARENCY MACROS
;----------------------

M16_InitGlobalVars:
  M16InitTrnsp := "255" ;Set initial transparency
                             ;Range 0 (transparent) to 255 (opaque)
  ;If you find transparency helpful, change M16InitTrnsp to a value
  ;between 180 and 200. Multiples of 20 work well here.
  M16CurrTrnsp := M16InitTrnsp ;Set to initial transparency
  M16MinTrnsp  := "60"  ;Minimum transparency level (transparent)
  M16MaxTrnsp  := "255" ;Maximum transparency level (opaque)
  M16IncTrnsp  := "20"  ;Amount to increment transparency level
  M16DecTrnsp  := "20"  ;Amount to decrement transparency level
Return

#IfWinActive, ahk_group SSGroup

;#M16a Decrement Opacity (make more transparent)
;Keys: Ctrl+Shift+Minus
^+-::
  If (M16CurrTrnsp >= M16MinTrnsp) ;must remain visible!
    {
    M16CurrTrnsp -= M16DecTrnsp
    If (M16CurrTrnsp < M16MinTrnsp)
      M16CurrTrnsp := M16MinTrnsp ;Must not fall below min
    Gosub, glbSetTrnsprncy
    }
Return

              ;----------------

;#M16b Increment Opacity (make more transparent)
;Keys: Ctrl+Shift+Plus
^++::
  If (M16CurrTrnsp <= M16MaxTrnsp)
    {
    M16CurrTrnsp += M16IncTrnsp
    If (M16CurrTrnsp > M16MaxTrnsp)
      M16CurrTrnsp := M16MaxTrnsp ;Must not exceed max
    Gosub, glbSetTrnsprncy
    }
Return

              ;----------------

;#M16c Reset to initial transparency
;Keys: Ctrl+Shift+PrtSc/SysRq
^+PrintScreen::
  M16CurrTrnsp := M16InitTrnsp ;Set to initial transparency
  Gosub, glbSetTrnsprncy
Return

              ;----------------

;#M16d Set to maximum opacity
;Keys: Ctrl+Shift+Break/Pause
^+CtrlBreak::
  M16CurrTrnsp := M16MaxTrnsp ;Set to maximum transparency
  Gosub, glbSetTrnsprncy
Return

;----------------------------


;=======================================================
;--------------------
;GLOBAL LABELS
;(common subroutines)
;listed alphabetically
;--------------------

glbActivateControlWindow:
  IfWinNotActive, SimSig ;SimSig's Control window is titled "SimSig - ..."
    WinActivate, SimSig
Return

glbCopyTDToClipboard:
  Click right ;open berth context menu
  SendInput i ;open Interpose window
  WinWaitActive, Interpose ;ensure it is open and on top
  SendInput ^c ;copy (already highlighted) TD to clipboard
  SendInput {Esc} ;cancel Interpose window and return to View
Return

;---

glbCreateStickyNoteWithClipboard:
  Click right ;open View context window
  SendInput a ;Hotkey to Add Sticky Note
Return

;---

glbSetTrnsprncy:
  IfWinExist, Train List
    WinSet, Transparent, %M16CurrTrnsp%, Train List
  IfWinExist, Show Timetable
    WinSet, Transparent, %M16CurrTrnsp%, Show Timetable
   ;Comment-out the above line before taking screenshots!
Return

;--------------------

